#!/usr/bin/env python3

import os
import sys
import time
import sqlite3


# quick sqlite3 concurrency check
# run multiple instances of this to verify timesharing/corruption
#
# macos [3.8.5, 2.7.17]:
#   no need to reopen connection or cursor, just conn.flush() is enough,
#   guarantee that the flushed data is available to other processes
# windows [3.8.1, 2.7.17, 3.4.4]:
#   same as macos


WINDOWS = sys.platform == "win32"
conn = None
c = None

if WINDOWS:
    os.system("rem")


def connect():
    global conn, c
    conn = sqlite3.connect('a.db')


connect()
c = conn.cursor()
me = os.getpid()

print('creating table')
try:
    c.execute('''create table b (k int primary key, n int)''')
except:
    pass


def put():
    # print('querying value')
    c.execute('select * from b')
    rows = c.fetchall()
    n = next((x[1] for x in rows if x[0] == me), None)
    if n is None:
        # print('inserting value')
        if False:
            # not supported in py3.4
            c.execute('''insert into b values (?,1) on conflict(k) do update set n=excluded.n''', (me,))
        else:
            # c.execute('''delete from b where k=?''', (me,))
            c.execute('''insert into b values (?,1)''', (me,))
        return 1, rows
    else:
        n = n + 1
        c.execute('''update b set n=? where k=?''', (n, me, ))
        return n, rows


pn = 0
pr = ""
fr = False
pt = time.time()
while True:
    n, rows = put()
    nt = time.time()
    dt = nt - pt
    pt = nt
    ct = "\033[1;37;45m" if dt > 0.02 else "\033[0m"

    dn = n - pn
    pn = n
    rows.sort()
    r = " ".join(str(x) for x in [n, dn, repr(rows), ct, dt, "\033[0m"])
    nfr = dn != 1 or dt > 0.02
    if nfr:
        print(pr)
    if fr or nfr or n % 10 == 0:
        print(r)
    pr = r
    fr = nfr
    
    if dn <= 0:
        raise Exception()

    conn.commit()
    if False:
        time.sleep(0.0002)
    
    if False:
        c.close()
        if False:
            conn.close()
            connect()
        
        c = conn.cursor()
